/*  APDUManager.cpp -- smartcard communication APDU oriented interface  
 
    This is part of SecureFW, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/securefw.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

#include "APDUManager.h"
#include "..\Shared\status.h"

//
//  PUBLIC
//
CAPDUManager::CAPDUManager() {
    m_hSession = NULL;
}

CAPDUManager::~CAPDUManager() {
    if (m_hSession) CloseSession();
}

int CAPDUManager::OpenSession(int slotID, char* acceptedCards) {
	int     status = STAT_OK;

    // CLOSE PREVIOUS SESSION, 
    if (m_hSession) CloseSession();

    // OPEN NEW SESSION
    status = TranslateCardCommError(cc_open(&m_hSession, slotID, acceptedCards));

    return status;
}

int CAPDUManager::CloseSession() {
    int     status = STAT_OK;

    // CLOSE ACTUAL SESION
    if (m_hSession) {
        status = TranslateCardCommError(cc_close(m_hSession));
        m_hSession = NULL;
    }

    return status;
}

    
int CAPDUManager::ExchangeAPDU(CARDAPDU* pAPDU) {
    int         status = STAT_OK;

    if (m_hSession) {
        
        // CLEAR SOFTWARE RETURN STATUS
        pAPDU->sw = SW_NO_ERROR;

        // SEND APDU
        if ((status = TranslateCardCommError(cc_apdu(m_hSession, pAPDU))) == STAT_OK) {
	        // OK
    
            // CHECK FOR SOFTWARE ERROR
            if (pAPDU->sw == SW_NO_ERROR) {
                // NO SOFWARE ERROR
            }            
            else {
                // CHECK FOR 'RESPONSE DATA AVAILABLE' STATUS
                if ((pAPDU->sw & 0xFF00) == SW_BYTES_REMAINING_00) {
                    if (pAPDU->lc == 0x00) {    // SYSTEM CALL TYPE (PROBABLY TO OBTAINING RESPONCE OUTPUT DATA)
                        // INSUFFICIENT OUTPUT DATA LENGTH
                        BYTE realLen = LOWBYTE(pAPDU->sw);						
                        pAPDU->le = realLen;

                        // NEW APDU WITH REQUIRED OUTPUT DATA LENGTH
                        status = ExchangeAPDU(pAPDU);    
                    }
                    else {
                        // USER CALL TYPE, OBTAIN RESPONSE DATA
		                BYTE        realLen = LOWBYTE(pAPDU->sw);				
                        CARDAPDU    apdu;

                        // PREPARE SYSTEM APDU FOR RECIEVE RESPONSE OUTPUT DATA 
		                apdu.cla = 0x00;
		                apdu.ins = 0xC0;
		                apdu.p1 = 0x00;
		                apdu.p2 = 0x00;
		                apdu.lc = 0x00;
                        apdu.le = realLen;
		                
                        if ((status = ExchangeAPDU(&apdu)) == STAT_OK) {
                            // COPY RECIEVED APDU
                            memcpy(pAPDU->DataOut, apdu.DataOut, apdu.le); 
                    
                            pAPDU->le = apdu.le;
                        }
	                }
                }
                else {
                    // CHECK FOR 'CORRECT DATA LENGTH' STATUS
                    if ((pAPDU->sw & 0xFF00) == SW_CORRECT_LENGTH_00) {
                        pAPDU->le = LOWBYTE(pAPDU->sw);
                    }
                    else {
                        // SOFTWARE ERROR OCCURED
                        status = TranslateISO7816Error(pAPDU->sw);
                    }
                }
            }
        }
    }
    else status = STAT_SESSION_NOT_OPEN;

    return status;
}    


//
//  PROTECTED
//
int CAPDUManager::TranslateCardCommError(int CardCommError) {
    switch (CardCommError) {
        case CCR_OK: return STAT_OK;
        case CCR_INVALID_HANDLE: return STAT_CCR_INVALID_HANDLE;
        case CCR_CARD_NOT_PRESENT: return STAT_CCR_CARD_NOT_PRESENT;
        case CCR_SESSION_NOT_OPENED: return STAT_CCR_SESSION_NOT_OPENED;
        case CCR_INVALID_SLOT_ID: return STAT_CCR_INVALID_SLOT_ID;
        case CCR_FUNCTION_FAILED: return STAT_CCR_FUNCTION_FAILED;
        case CCR_CANCELLED: return STAT_CCR_CANCELLED;
        default: return STAT_CCR_UNKNOWN;
    }

    return STAT_CCR_UNKNOWN;
}

int CAPDUManager::TranslateISO7816Error(int iso7816Error) {
    return iso7816Error;
}
